""" Search method with retrieval-reranking
"""
import time
import numpy as np
from semanticsearch import utils 

LOGGER = utils.init_logger()

def search(query,index,bi_encoder,cross_encoder,passages):
    LOGGER.info(f"Input question: {query}")

    ##### Sematic Search #####
    # Encode the query using the bi-encoder and find potentially relevant passages
    t=time.time()
    query_vector = bi_encoder.encode([query])
    top_k = index.search(query_vector, 3)
    top_k_ids = top_k[1].tolist()[0]
    top_k_ids = list(np.unique(top_k_ids))
    LOGGER.info('>>>> Results in Total Time: {}'.format(time.time()-t))

    ##### Re-Ranking #####
    # Now, score all retrieved passages with the cross_encoder
    t=time.time()
    cross_inp = [[query, passages[hit]] for hit in top_k_ids]
    bienc_op=[passages[hit] for hit in top_k_ids]
    cross_scores = cross_encoder.predict(cross_inp)
    LOGGER.info('>>>> Results in Total Time: {}'.format(time.time()-t))

    # Output of top-5 hits from bi-encoder
    LOGGER.info("\n-------------------------\n")
    LOGGER.info("Top-3 Bi-Encoder Retrieval hits")
    for result in bienc_op:
        LOGGER.info("\t{}".format(result.replace("\n", " ")))
        
    # Output of top-5 hits from re-ranker
    LOGGER.info("\n-------------------------\n")
    LOGGER.info("Top-3 Cross-Encoder Re-ranker hits")
    results=[]
    for hit in np.argsort(np.array(cross_scores))[::-1]:
        LOGGER.info("\t{}".format(bienc_op[hit].replace("\n", " ")))
        results.append(bienc_op[hit].replace("\n", " "))

    json_resp={}
    for rank,result in enumerate(results):
        json_resp[f'rank_{rank+1}']=result

    return json_resp


import os

from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="semantic_search_faiss",
    version="0.0.9",
    author="Nandhagopalan Elangovan",
    description="Semantic search to query covid related papers",
    long_description_content_type="text/markdown",
    long_description=read("README.md"),
    license="MIT",
    keywords=["natural language processing", "semantic search", "pytorch"],
    url="https://github.com/Nandhagopalan/Semanticsearch",
    packages=find_packages(exclude=["tests", "docs", "images"]),
    install_requires=[
       "certifi==2021.5.30",
        "chardet==4.0.0",
        "click==8.0.1",
        "dataclasses",
        "faiss-cpu==1.7.1",
        "filelock==3.0.12",
        "huggingface-hub==0.0.8",
        "idna==2.10",
        "importlib-metadata==4.5.0",
        "joblib==1.0.1",
        "nltk==3.6.2",
        "numpy==1.19.5",
        "packaging==20.9",
        "pandas==1.1.5",
        "Pillow==8.2.0",
        "pyparsing==2.4.7",
        "python-dateutil==2.8.1",
        "pytz==2021.1",
        "regex==2021.4.4",
        "requests==2.25.1",
        "sacremoses==0.0.45",
        "scikit-learn==0.24.2",
        "scipy==1.5.4",
        "sentence-transformers==1.2.0",
        "sentencepiece==0.1.95",
        "six==1.16.0",
        "threadpoolctl==2.1.0",
        "tokenizers==0.10.3",
        "torch==1.8.1",
        "torchvision==0.9.1",
        "tqdm==4.61.0",
        "transformers==4.6.1",
        "typing-extensions==3.10.0.0",
        "urllib3==1.26.5",
        "zipp==3.4.1",
    ],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
